import { world, ItemStack, ItemTypes, system } from '@minecraft/server';

system.runInterval(js7_boots_of_swiftness_set, 1); 

world.afterEvents.entityDie.subscribe((event) => {
    const entity = event.deadEntity;
    const mobs = new Set([
        "minecraft:pillager",
        "minecraft:vindicator",
        "minecraft:evocation_illager"
    ]);

    if (!mobs.has(entity.typeId)) return;

    const { x, y, z } = entity.location;
    const dim = entity.dimension;

   
    const chance = (percent) => Math.random() < percent / 100;

    //10%
    if (chance(10)) {
        dim.spawnItem(new ItemStack(ItemTypes.get("js7_swiftg:boots_of_swiftness"), 1), { x, y, z });
    }
});

function js7_boots_of_swiftness_set() {
    const players = world.getPlayers();
    for (const player of players) {
        const equipment = player.getComponent("equippable");
     
        if (equipment == null) {
           
            continue;
        }
      
        const feetEquipment = equipment.getEquipment("Feet");
        
        const hasFeetEquipment = feetEquipment?.typeId === "js7_swiftg:boots_of_swiftness"; 
        
        if (hasFeetEquipment && player.isSprinting) {
        player.runCommandAsync(`effect @s speed 1 2 true`);
        player.runCommandAsync(`effect @s slow_falling 1 2 true`);
        }
    }
}

const Vector = {
    add: (a, b) => ({ x: a.x + b.x, y: a.y + b.y, z: a.z + b.z }),
    subtract: (a, b) => ({ x: a.x - b.x, y: a.y - b.y, z: a.z - b.z }),
    multiply: (a, scalar) => ({ x: a.x * scalar, y: a.y * scalar, z: a.z * scalar }),
};

// Define your two iron golem patterns
const IPatterns = [
    [
        
        { offset: { x: 0, y: -1, z: 0 }, ids: ['minecraft:iron_block'] },
        { offset: { x: 1, y: -1, z: 0 }, ids: ['minecraft:iron_block'] },
        { offset: { x: -1, y: -1, z: 0 }, ids: ['minecraft:iron_block'] },
        { offset: { x: 0, y: -2, z: 0 }, ids: ['minecraft:iron_block'] },
        { offset: { x: 1, y: 0, z: 0 }, ids: ['minecraft:air'] },
        { offset: { x: -1, y: 0, z: 0 }, ids: ['minecraft:air'] },
        { offset: { x: 1, y: -2, z: 0 }, ids: ['minecraft:air'] },
        { offset: { x: -1, y: -2, z: 0 }, ids: ['minecraft:air'] },
    ],
    [
        { offset: { x: 0, y: -1, z: 0 }, ids: ['minecraft:iron_block'] },
        { offset: { x: 0, y: -1, z: 1 }, ids: ['minecraft:iron_block'] },
        { offset: { x: 0, y: -1, z: -1 }, ids: ['minecraft:iron_block'] },
        { offset: { x: 0, y: -2, z: 0 }, ids: ['minecraft:iron_block'] },
        { offset: { x: 0, y: 0, z: 1 }, ids: ['minecraft:air'] },
        { offset: { x: 0, y: 0, z: -1 }, ids: ['minecraft:air'] },
        { offset: { x: 0, y: -2, z: 1 }, ids: ['minecraft:air'] },
        { offset: { x: 0, y: -2, z: -1 }, ids: ['minecraft:air'] },
    ],
];

// Pattern checker
function checkBlockPattern(baseBlock, pattern) {
    const shape = [];

    for (const entry of pattern) {
        const pos = Vector.add(baseBlock.location, entry.offset);
        const testBlock = baseBlock.dimension.getBlock(pos);
        if (!testBlock || !entry.ids.includes(testBlock.typeId)) return null;
        shape.push(testBlock);
    }

    return shape;
}

world.afterEvents.playerPlaceBlock.subscribe(({ player, block, dimension }) => {
    if (block.typeId !== 'js7_swiftg:boots_of_swiftness') return;

    for (const pattern of IPatterns) {
        const shape = checkBlockPattern(block, pattern);
        if (!shape) continue;

        // Clear structure blocks
        block.setType('minecraft:air');
        for (const b of shape) {
            b?.setType('minecraft:air');
        }

        const groundX = block.location.x + 0.5;
        const groundY = shape.reduce((a, b) => a < b.location.y ? a : b.location.y, block.location.y);
        const groundZ = block.location.z + 0.5;
        const ground = { x: groundX, y: groundY, z: groundZ };

        dimension.spawnEntity('js7_swiftg:swiftness_iron_golem<js7_swiftg:is_player_spawned>', ground);
        dimension.playSound('block.iron_block.break', ground);
        dimension.playSound('block.iron_block.break', ground);
        dimension.playSound('block.iron_block.break', ground);
        dimension.playSound('block.iron_block.break', ground);
        break;
    }
});



  world.afterEvents.entityHitEntity.subscribe(event => {
    try {
        const attacker = event.damagingEntity;
        const victim = event.hitEntity;

        if (!attacker || !victim) return; 

        if (attacker.typeId === "js7_swiftg:swiftness_iron_golem") {
            const entity_range = victim.location;
            const victimDimension = victim.dimension;
            let entitiesIn = victimDimension.getEntities({
                excludeTypes: ["js7_swiftg:swiftness_iron_golem","iron_golem","villager","item","xp_orb","lightning_bolt","evocation_fang","tnt","arrow"],
                excludeFamilies: ["inanimate"],
                location: entity_range,
                minDistance: 0.0,
                maxDistance: 1.25
            });

            for (const entityIn of entitiesIn) {
                entityIn.applyDamage(6, { cause: "contact", damagingEntity: attacker });
                const direction = attacker.getViewDirection();
                const direction_x = direction.x;
                const direction_z = direction.z;
                const verticalImpulse = 0.2;
                if (entityIn.typeId === "minecraft:player") {
                    entityIn.applyKnockback(2 * direction_x, 2 * direction_z, 2.5, 1.1);
                } else {
                    entityIn.applyImpulse({ x: 1 * direction_x, y: 4.5 * verticalImpulse, z: 1 * direction_z });
                }
            }
        }
    }catch(error){}finally{}
});

const mobConfigs = {

    "js7_swiftg:swiftness_iron_golem": {
        typeId: "js7_swiftg:swiftness_iron_golem",
        impulseY: 0.5,
        knockbackHorizontal: 1.25,
        knockbackVertical: 0.75,
        randomKnockback: false,
    },
};
world.afterEvents.entityHurt.subscribe((event) => {
    const damagingEntity = event.damageSource.damagingEntity;
    const hurtEntity = event.hurtEntity;
    const hurtCause = "swift_golem_arm_spin"; // Replace with the actual cause value from the event

    // Early exit if entities are missing
    if (!damagingEntity || !hurtEntity)
        return;
    // Check if the entities are still valid
    if (!damagingEntity.isValid() || !hurtEntity.isValid()) {
        return;
    }
    // Define a set of valid entity types that support getViewDirection
    const validEntityTypes = new Set([
     
        "js7_swiftg:swiftness_iron_golem",
        // Add other entity types that should have getViewDirection
    ]);
    // Only proceed if the entity type is in the valid set
    if (!validEntityTypes.has(damagingEntity.typeId)) {
        return; // Exit if the entity type is not valid
    }
    // Get the view direction of the damaging entity
    const direction = damagingEntity.getViewDirection();
    const direction_x = direction.x;
    const direction_z = direction.z;
    const mobConfig = mobConfigs[damagingEntity.typeId];
    if (!mobConfig)
        return; // If mob is not configured, exit
    // Utility function to apply impulse or knockback
    function applyImpulseOrKnockback(entity, x, z, y, horizontal, vertical) {
       
     
        // Check if the entity is still valid
        if (damagingEntity.hasTag("js7_swiftg:swift_golem_arm_spin")) {
        if (!entity.isValid()) {
            return;
        }
        if (entity.typeId === "minecraft:player") {
            entity.applyKnockback(x, z, horizontal, vertical);
            entity.applyDamage(9, { cause: "contact", damagingEntity: damagingEntity });
        }
        else {
            entity.applyDamage(9, { cause: "contact", damagingEntity: damagingEntity });
            entity.applyImpulse({ x, y, z });
        }
    }
}
    if (mobConfig.randomKnockback) {
        const variable_horizontal = Math.random() * (6.7 - 4.4) + 4.4; // Random value between 4.4 and 6.7
        const variable_vertical = Math.random() / 7; // Random value between 0 and 1
        hurtEntity.applyKnockback(direction_x, direction_z, variable_horizontal, variable_vertical);
        return;
    }
    if (mobConfig.radiusSquared !== undefined) {
        const position_of_mob = damagingEntity.location;
        const position_of_target = hurtEntity.location;
        // Calculate the distance in 3D space
        const deltaX = position_of_target.x - position_of_mob.x;
        const deltaY = position_of_target.y - position_of_mob.y;
        const deltaZ = position_of_target.z - position_of_mob.z;
        const distanceSquared = deltaX * deltaX + deltaY * deltaY + deltaZ * deltaZ;
        // Check if the target is within the radius in 3D space
        if (distanceSquared <= mobConfig.radiusSquared) {
            applyImpulseOrKnockback(hurtEntity, direction_x, direction_z, mobConfig.impulseY, mobConfig.knockbackHorizontal, mobConfig.knockbackVertical);
        }
    }
    else {
        applyImpulseOrKnockback(hurtEntity, direction_x, direction_z, mobConfig.impulseY, mobConfig.knockbackHorizontal, mobConfig.knockbackVertical);
    }
});

